@@ -1033,7 +1033,6 @@ class _ExprTree(list):
     """
     Tree of expression
     """
-    hasher = repr
 
     def __init__(self, content):
         list.__init__(self, content)
@@ -1141,12 +1140,6 @@ class _ExprTree(list):
             end += 1
         return slice(begin, end)
 
-    def __hash__(self):
-        return hash(repr(self))
-
-    def __eq__(self, other):
-        return hash(self) == hash(other)
-
     def top(self):
         """accumulative operation"""
         return self[::2]
@@ -1161,6 +1154,7 @@ class _ExprTree(list):
         self.clear()
         self.extend(new_inds)
 
+
 class SymbolTree(_ExprTree):
     """ Individual Tree, each tree is one expression"""
 
@@ -1191,7 +1185,10 @@ class SymbolTree(_ExprTree):
         return _ExprTree.__str__(self)
 
     def __hash__(self):
-        return hash(self.hasher(self))
+        return hash(tuple(self))
+
+    def __eq__(self, other):
+        return repr(self) == repr(other)
 
     def compress(self):
 
@@ -1535,18 +1532,18 @@ class CalculatePrecisionSet(SymbolSet):
     def try_add_coef_times(self, expr, grid_x=None):
 
         pre_y_all_expr01 = try_add_coef_times(expr, self.data_x, self.y, self.terminals_and_constants_repr, grid_x,
-                                               filter_warning=self.filter_warning, inter_add=self.inter_add,
-                                               inner_add=self.inner_add, vector_add=self.vector_add,
-                                               out_add=self.out_add,
-                                               flat_add=self.flat_add,
-                                               np_maps=self.np_map, classification=self.classification,
-                                               random_state=0,
-                                               return_expr=False
-                                               )
+                                              filter_warning=self.filter_warning, inter_add=self.inter_add,
+                                              inner_add=self.inner_add, vector_add=self.vector_add,
+                                              out_add=self.out_add,
+                                              flat_add=self.flat_add,
+                                              np_maps=self.np_map, classification=self.classification,
+                                              random_state=0,
+                                              return_expr=False
+                                              )
 
         return pre_y_all_expr01
 
-    def parallelize_try_add_coef_times(self, exprs, grid_x=None,resample_number=500):
+    def parallelize_try_add_coef_times(self, exprs, grid_x=None, resample_number=500):
         if isinstance(grid_x, np.ndarray):
             grid_x = list(grid_x.T)
         calls = functools.partial(try_add_coef_times, x=self.data_x, y=self.y,
