@@ -3,9 +3,9 @@ import time
 from numpy import random
 from sklearn.datasets import load_boston
 
-from BGP.base import SymbolSet, SymbolTree
-from BGP.calculation.translate import compile_context
-from BGP.functions.dimfunc import dless, Dim
+from bgp.base import SymbolSet, SymbolTree
+from bgp.calculation.translate import compile_context
+from bgp.functions.dimfunc import dless, Dim
 
 if __name__ == "__main__":
     # data
